# originally in lec 27
class LinkedList:
    """Implements our own recursive list data structure"""
    __slots__ = ['_value', '_rest']

    def __init__(self, v=None, r=None):
        # call setters for _value and _rest
        self.setValue = v
        self.setRest = r

    @property
    def getRest(self):
        return self._rest

    @property
    def getValue(self):
        return self._value

    @getRest.setter
    def setRest(self, val):
        self._rest = val

    @getValue.setter
    def setValue(self, val):
        self._value = val

    def __len__(self):
        # base case: i'm last item
        if self.getRest is None:
            return 1
        else:
            # same as return 1 + self.rest.__len__()
            return 1 + len(self.getRest)

    def __contains__(self, val):
        if self.getValue == val:
            return True
        elif self.getRest is None:
            return False
        else:
            return val in self.getRest

    def __eq__(self, other):
        # If both lists are empty
        if self.getRest is None and other.getRest is None:
                return True

        # If both lists are not empty, then data of
        # current nodes must match, and same should
        # be recursively true for rest of the nodes.
        elif self.getRest is not None and other.getRest is not None :
            return self.getValue == other.getValue and self.getRest == other.getRest

        # If we reach here, then one of the lists
        # is empty and other is not
        return False

    def __str__(self):
        # can also do return "value: {}, next: {}".format(self.value, self.rest)
        # format calls str
        return "value : " + str(self.getValue) + ", next : (" + str(self.getRest) + ")"



if __name__ == "__main__":
    myList = LinkedList("a")
    myList = LinkedList("b", myList)
    myList = LinkedList("c", myList)

    print("myList:", myList)
    print("value:",myList.getValue)
    print("rest:", myList.getRest)

    print("Length:", len(myList))
    print("c in list?", "c" in myList)
    print("d in list?", "d" in myList)

    myList2 = LinkedList("c", LinkedList("b", LinkedList("a")))
    print("myList2:", myList2)

    print("myList == myList2?", myList2 == myList)
