# originally in lec 27
class LinkedList:
    """Implements our own recursive list data structure"""
    __slots__ = ['_value', '_rest']

    def __init__(self, value=None, rest=None):
        # call setters for value and rest
        self.value = value
        self.rest = rest

    @property
    def rest(self):
        return self._rest

    @property
    def value(self):
        return self._value

    @rest.setter
    def rest(self, val):
        self._rest = val

    @value.setter
    def value(self, val):
        self._value = val

    def __len__(self):
        # base case: i'm last item
        if self.rest is None:
            return 1
        else:
            return 1 + len(self.rest)

    def __contains__(self, val):
        if self.value == val:
            return True
        elif self.rest is None:
            return False
        else:
            return val in self.rest

    def __eq__(self, other):
        # If both lists are empty
        if self.rest is None and other.rest is None:
            return True

        # If both lists are not empty, then data of current elements must match,
        # and same should be recursively true for rest of the elements
        elif self.rest is not None and other.rest is not None :
            return self.value == other.value and self.rest == other.rest

        # If one of the lists is empty and other is not
        else:
            return False

    def __str__(self):
        return "value : " + str(self.value) + ", next : (" + str(self.rest) + ")"

    def __repr__(self):
        return "LinkedList('{}',{})".format(self.value, self.rest)


###### remove the following??

    def __add__(self, other):
        if self.rest is None:
            self._rest = other
        else:
            self.rest.__add__(other)
        return self


    def append(self, val):
        if self.rest is None:
            self._rest = LinkedList(val)
        else:
            self.rest.append(val)


    def __getitem__(self, index):
        if index == 0:
            return self.value
        else:
            return self.rest[index - 1]


    def __setitem__(self, index, val):
        if index == 0:
            self._value = val
        else:
            self.rest[index - 1] = val


    def __eq2__(self, other):
        # while both have elements, compare and move one forward
        while self and other and self.value == other.value:
            self = self.rest
            other = other.rest
        # if both are empty (and same so far)
        if not self and not other:
            return True
        # otherwise
        return False


if __name__ == "__main__":
    myList = LinkedList("a")
    myList = LinkedList("b", myList)
    myList = LinkedList("c", myList)

    print("myList:", myList)
    print("value:",myList.value)
    print("rest:", myList.rest)

    print("Length:", len(myList))
    print("c in list?", "c" in myList)
    print("d in list?", "d" in myList)
    print("myList[1]:", myList[1])

    myList2 = LinkedList("c", LinkedList("b", LinkedList("a")))
    print("myList2:", myList2)

    print("myList == myList2?", myList2 == myList)

    myList3 = myList + myList2
    print("myList3:", myList3)
    print("myList2 == myList3?", myList2 == myList3)
    print("myList == myList3?", myList3 == myList)


    myList3[1] = "d"
    print("myList3 with d:", myList3)

    myList3.append("e")
    print("myList3 with e:", myList3)
